package tests.part2;
/*********************************************************************************************
//* Filename:		Test4_File_SystemKey_With_Compliance.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARFile;
import com.clear.cryptosystem.CLEARLicenseInfo;
import com.clear.cryptosystem.CLEARMode.StreamingDecryptionMode;
import com.clear.cryptosystem.CLEARMode.StreamingEncryptionSysGenKeyMode;
import com.clear.cryptosystem.CLEARResult;

import tools.EvaluateResult;
import tools.FileIO;
import tools.InspectResult;
import tools.Stopwatch;
import tools.TestCleanup;

public class Test4_File_SystemKey_With_Compliance {
	
	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 	= true;  // SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	
	/**
	 * UNIT TEST BLOCK
	 * @param File input
	 * @param File duplicate
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean test(File input, File duplicate) throws Exception {
		
		boolean pass = false;
		
		System.out.println("RUN UNIT TEST: [" + Test4_File_SystemKey_With_Compliance.class.getCanonicalName() + "]");
		
		// STOPWATCH - COMPARE TIMES FOR ENCRYPTION / DECRYPTION - DIFFERENT STRENGTH LEVELS
		Stopwatch sw = new Stopwatch();
		
		
		// INITIALIZE "FILE" INTERFACE
		CLEARFile cf = CLEAR.clearFile();			// FACTORY PATTERN - GET A REFERENCE TO CLEAR API (STRING ENCRYPTION / DECRYPTION INTERFACE)
		
		
		
		// ENCRYPT A LOCAL FILE - SYSTEM GENERATED KEY
		sw.setTimeMark(1);
		CLEARResult cr = cf.encrypt(duplicate.getAbsolutePath(), CLEAR.STRENGTH_512_BIT, StreamingEncryptionSysGenKeyMode.STREAMING_SYSTEM_GEN_KEY_WITH_COMPLIANCE);
		sw.setTimeMark(2);
		
		
		// INSEPCT OUTPUT OF ENCRYPTION
		InspectResult.inspectEncryptionResultsFile(cr, sw, duplicate.getAbsolutePath(), Test4_File_SystemKey_With_Compliance.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		
		// *** PAUSE & CLEANUP *** - ALLOW TIME TO VISUALLY INSPECT OUTPUT OF ENCRYPTION ON DESKTOP / FOLDER / ETC.
		duplicate.delete();
		Thread.sleep(5000);  // WAIT [FIVE] SECONDS TO OBSERVE THAT THE FILE HAS BEEN REMOVED - LOCALLY
		
		
		
		// DECRYPT A FILE - WITH THE SYSTEM-GENERATED KEYSTRING PRODUCED BY ENCRYPTION
		sw.setTimeMark(3);
		CLEARResult cr2 = cf.decrypt(cr.getCipherTextFile(), cr.getKeyFile(), StreamingDecryptionMode.STREAMING_KEY_WITH_COMPLIANCE);
		sw.setTimeMark(4);
		
		
		
		// INSPECT OUTPUT OF DECRYPTION
		InspectResult.inspectDecryptionResultsFile(cr2, sw, input.getAbsolutePath(), Test4_File_SystemKey_With_Compliance.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		// EVALUATE DECRYPTION RESULT
		pass = EvaluateResult.evaluateFileDecryptionResult(cr2, input.getAbsolutePath(), Test4_File_SystemKey_With_Compliance.class.getCanonicalName());
		
		
		// CLEANUP GENERATED TEST FILES - REMOVES GENERATED KEY AND GENERATED CIPHERTEXT
		TestCleanup.cleanup(cr);
		
		
		return pass;
	}
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	 * @return void
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			EvaluateResult.set_DISPLAY_TEST_RESULTS(_DISPLAY_RESULTS_TO_CONSOLE);
			InspectResult.set_DISPLAY_RESULTS_TO_CONSOLE(_DISPLAY_RESULTS_TO_CONSOLE);
			
			if ((args != null) && (args.length > 0) && (args[0] != null)) {
				
				File input = new File(args[0]);
				
				if ((input != null) && (input.exists())) {
					
					// MAKE A BACKUP COPY OF THE INPUT FILE SO THAT IT CAN BE USED DURING TESTING
					String[] parts = FileIO.getFileExtensionParts(input.getAbsolutePath());
					File duplicate = new File(parts[0] + "_copy" + parts[1]);
					FileIO.copyfile(input, duplicate);
					
					test(input, duplicate); 	 // EXECUTE UNIT TEST
				}
				else {
					System.out.println("Could not locate test file:  " + args[0]);
				}
			}
			else {
				System.out.println("This unit test requires program input parameters");
			}
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
