package tests.part3;
/*********************************************************************************************
//* Filename:		Test3_Streaming_Advanced_PacketLoss_Recovery.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.Serializable;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARLicenseInfo;
import com.clear.cryptosystem.CLEARMode.StreamingDecryptionMode;
import com.clear.cryptosystem.CLEARMode.StreamingEncryptionSysGenKeyMode;
import com.clear.cryptosystem.CLEARResult;
import com.clear.cryptosystem.CLEARStream;

import tools.Stopwatch;


public class Test3_Streaming_Advanced_PacketLoss_Recovery {

	private static String _SYMMETRIC_KEY_EXCHANGE; 							// SET DURING TRANSMISSION PHASE
	
	private static final int[] _DROP_PACKETS = { 6, 7, 8, 9, 14 }; 			// SIMULATED FAILURE -> DROP THESE PACKETS!

//	private static final int[] _DROP_PACKETS = { 6, 7, 8 }; 				// SIMULATED FAILURE -> DROP THESE PACKETS!
//	private static final int[] _DROP_PACKETS = { 6, 7}; 					// SIMULATED FAILURE -> DROP THESE PACKETS!
//	private static final int[] _DROP_PACKETS = { 6 }; 						// SIMULATED FAILURE -> DROP THESE PACKETS!
	
	
	private static final boolean _ENABLE_AUTO_PACK_DROP_RECOVERY 	= true;
	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 		= true; 	// SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	
	/**
	 * UNIT TEST BLOCK
	 * @throws Exception
	*/
	public static void test() throws Exception {
		
		
		System.out.println("RUN UNIT TEST: [" + Test3_Streaming_Advanced_PacketLoss_Recovery.class.getCanonicalName() + "]");
		
		// STOPWATCH - COMPARE TIMES FOR ENCRYPTION / DECRYPTION - DIFFERENT STRENGTH LEVELS
		Stopwatch sw = new Stopwatch();
		
		
		// GENERATE MOCKING DATA - SRC
		List<byte[]> src = generateMockingDataSRC();  	// GET MOCKING DATA
		
		
		// ENCRYPT "PACKETS" OF DATA TO BE STREAMED -> (I.E;  SOCKET SERVER, ETC.)
		sw.setTimeMark(1);
		ArrayDeque<PacketVO> encryptedPackets = encryptPackets(src);
		sw.setTimeMark(2);
		inspectEncryptionResults(src, encryptedPackets, sw);  // DIAGNOSTIC TESTING ONLY
		
		
		
		// DECRYPT "PACKETS" OF DATA FROM STREAM 
		sw.setTimeMark(3);
		List<byte[]> decryptedPackets = decryptPackets(encryptedPackets);
		sw.setTimeMark(4);
		inspectDecryptionResults(src, decryptedPackets, sw);  // DIAGNOSTIC TESTING ONLY
		
		
		
		// CHECK DECRYPTION RESULT
		evaluateUnitTestResult(src, decryptedPackets);
	}

	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** >> ENCRYPTION IMPLEMENTATION BELOW THIS LINE : 																																								  				  **/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	
	/**
	 * ATOMIC SUBROUTINE - ENCRYPT STREAMING DATA PACKETS
	 * @param List<byte[]> src
	 * @return ArrayDeque<PacketVO> 
	*/
	private static ArrayDeque<PacketVO> encryptPackets(List<byte[]> src) {
		
		// INITIALIZE "STREAM" INTERFACE
		CLEARStream cs = CLEAR.clearStream();			// FACTORY PATTERN - GET A REFERENCE TO CLEAR API (STRING ENCRYPTION / DECRYPTION INTERFACE)

		
		// INSTANTIATE COLLECTIONS / MOCKING DATA
		ArrayDeque<PacketVO> packets = new ArrayDeque<PacketVO>();
		
		
		// VARIABLES TO CONTROL STREAMING ENCRYPTION JOB
		int c = 0;
		long jobID = 0;
		boolean terminate = false;
		String keyString = null;
		CLEARResult cr = null;
				
		int strength = CLEAR.STRENGTH_512_BIT;
//		int strength = CLEAR.STRENGTH_1024_BIT;
		
		
		// LOOP THROUGH STREAMING DATA (SYNTHESIZED "PACKET" DATA)
		for (byte[] data : src) {
			
			
			// CHECK FOR END OF PACKET STREAM - CLOSE ENCRYPTION JOB AT END OF SRC DATA
			if (c == (src.size() -1)) { 
				terminate = true; // CAUSES ENCRYPTION TO END
			}
			
			
			// DO ENCRYPTION!  - FIRST PACKET INITIALIZES JOB - SUBSEQUENT PACKETS CONTINUE ON "JOB-ID"
			if (c == 0) {
				
				// BEGIN A NEW STREAMING JOB (FIRST PACKET) - PERFORMS INIT
				cr = cs.encryptStart(data, strength, StreamingEncryptionSysGenKeyMode.STREAMING_SYSTEM_GEN_KEY, terminate);  // ENCRYPT A PACKET
				jobID = cr.getJobNumber(); // KEEP TRACK OF STREAMING JOB NUMBER FOR NEXT ITERATION
			}
			else {
				
				// CONTINUE EXISTING STREAMING BASED ON ** JOB-ID ** - (NN PACKETS) 
				cr = cs.encryptContinue(data, jobID, StreamingEncryptionSysGenKeyMode.STREAMING_SYSTEM_GEN_KEY, terminate);
			}
			
			// PARSE SYSTEM-GENERATED KEY MATERIAL FROM INITIALIZATION SEGMENT
			keyString = (keyString == null ? cr.getKeyMaterial() : keyString);
			
			
			// PERFORM SYMMETRIC PRIVATE-KEY EXCHANGE - THIS UNIT (ALICE) NEEDS TO TRANSMIT A KEY TO BOB
			transmitKeyExchange(keyString);  // TRANSMIT KEY TO BOB
			
			
			// CAPTURE ENCRYPTED PACKET DATA TO SEND ONWARD TO RECEIVER
			PacketVO packetVO = new PacketVO();
			packetVO.setData(cr.getCipherText());
			packetVO.setSequenceNumber(c + 1);
			packets.add(packetVO);
			
			
			// System.out.println("E" + (c+1) + " ORIGINAL [" + new String(data) + "]");
			
			c++;
		}
		
		return packets;
	}
	
	/**
	 * ATOMIC SUBROUTINE - SYNTHESIZE A KEY EXCHANGE
	 * @param keyString
	*/
	private static void transmitKeyExchange(String keyString) {
		
		if (_SYMMETRIC_KEY_EXCHANGE == null) {
			_SYMMETRIC_KEY_EXCHANGE = keyString;
		}
	}
	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** >> DECRYPTION IMPLEMENTATION BELOW THIS LINE : 																																								  				  **/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	
	/**
	 * ATOMIC SUBROUTINE - DECRYPT STREAMING DATA PACKETS
	 * @parm ArrayDeque<PacketVO> packets
	 * @return List<byte[]>
	*/
	private static List<byte[]> decryptPackets(ArrayDeque<PacketVO> packets) {
		
		// PULL FROM STREAMING FAUX NETWORK SOURCE - DECRYPT EACH BYTE AS WE RECEIVE THEM.  
		// ** SIMULATES RECEIVER **
		List<byte[]> decryptedPackets = new ArrayList<byte[]>();
		
		
		// INITIALIZE "STREAM" INTERFACE
		CLEARStream cs = CLEAR.clearStream();			// FACTORY PATTERN - GET A REFERENCE TO CLEAR API (STRING ENCRYPTION / DECRYPTION INTERFACE)
		
		
		// VARIABLES TO CONTROL STREAMING DECRYPTION JOB
		int p = 0;
		long jobID = 0;
		int numberOfDroppedPackets = 0;
		int totalReceived = packets.size();
		boolean terminate = false; // reset flag
		CLEARResult cr = null;
		
		
		// PERFORM SYMMETRIC PRIVATE-KEY EXCHANGE - THIS UNIT (BOB) NEEDS TO RECEIVE A KEY FROM ALICE
		String keyString = _SYMMETRIC_KEY_EXCHANGE;
		
		
		// LOOP THROUGH STREAMING DATA (SYNTHESIZED "PACKET" DATA)
		while (!packets.isEmpty()) {
			
			
			PacketVO packet = packets.removeFirst();  // TAKE NEXT AVAILABLE PACKET OF ENCRYPTED DATA (CIPHERTEXT)
			
			int packetSize = packet.getData().length;
			
			if (p == (totalReceived -1)) { 
				terminate = true; // CAUSES DECRYPTION TO END (GENERATES HMAC SIGNATURE WHEN IN HMAC MODE)
			}
			
			
			// DO DECRYPTION!  - FIRST PACKET INITIALIZES JOB - SUBSEQUENT PACKETS CONTINUE ON "JOB-ID"
			if (p == 0) {
				
				// BEGIN A NEW STREAMING JOB (FIRST PACKET) - PERFORMS INIT
				cr = cs.decryptStart(packet.getData(), keyString, StreamingDecryptionMode.STREAMING_KEY, terminate);
				jobID = cr.getJobNumber();  // KEEP TRACK OF STREAMING JOB NUMBER FOR NEXT ITERATION
			}
			else {
				
				
				/** ** ** ** INJECT SIMULATED PACKET LOSS HERE! ** ** ** **/
				if (simulateDropPacket(packet)) {  // SIMULATE PACKET-LOSS TYPE OF FAILURE SCENARIO
					p++; // increment process counter
					numberOfDroppedPackets++; // increment count number of drops
					continue;  // SKIP THIS DECRYPTION PROCESS (PUTS DECRYPTION STATE OUT OF SYNC)
				}
				/** ** ** ** INJECT SIMULATED PACKET LOSS HERE! ** ** ** **/
				
				
				
				/** ** ** ** INJECT SIMULATED DROP-PACKET RECOVERY HERE! ** ** ** **/
				// SIMULATE PACKET-RECOVERY [1 PACKETS] AFTER THE DROP - NOTE - THIS COULD BE *ANY* NUMBER OF PACKETS / INTERVAL AFTER A DROP
				if (numberOfDroppedPackets > 0) {
					
					if (_ENABLE_AUTO_PACK_DROP_RECOVERY) {
						
						cs.ff(jobID, numberOfDroppedPackets, packetSize);
						
						numberOfDroppedPackets = 0;  // reset dropped packet count
						
						System.out.println("RE-ALIGNED CRYPTOGRAPHIC STREAM CYCLE");
					}
				}
				/** ** ** ** INJECT SIMULATED DROP-PACKET RECOVERY HERE! ** ** ** **/
				
				
				// CONTINUE EXISTING STREAMING BASED ON ** JOB-ID ** - (NN PACKETS) 
				cr = cs.decryptContinue(packet.getData(), jobID, StreamingDecryptionMode.STREAMING_KEY, terminate);
			}
			
			
			// CAPTURE DECRYPTED (CLEARTEXT) PACKET(S)
			decryptedPackets.add(cr.getClearText());
			
			p++;
		}
		
		return decryptedPackets;
	}
	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** >> UNIT TEST [PASS / FAIL] EVALUATION BELOW THIS LINE : 																																									  **/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	
	
	
	/**
	 * ATOMIC SUBROUTINE - EXAMINE ENCRYPTION RESULTS FOR EXPECTED OUTPUT - DISPLAYS PASS OR FAIL
	 * @param List<byte[]> src
	 * @param List<byte[]> decryptedPackets
	 * @return void
	 * @throws Exception
	*/
	private static void evaluateUnitTestResult(List<byte[]> src, List<byte[]> decryptedPackets) throws Exception {
		
		boolean pass = false;
		
		if ((src != null) && (decryptedPackets != null)) {
			
			int discrepencies = 0;
			
			int drops = 0;
			
			for (int i = 0; i < src.size(); i++) {
				
				if (_ENABLE_AUTO_PACK_DROP_RECOVERY && isDroppedPacketSimulation((i + 1))) {
					drops++;
					continue;
				}
				
				byte[] original  = src.get(i);
				byte[] decrypted = decryptedPackets.get((i-drops));
				
				
				System.out.println("*P" + (i+1) + "* ORIGINAL [" + new String(original) + "] -- *D" + ((i+1)-drops) + "* DECRYPTED [" + new String(decrypted) + "]");
				
				boolean equivalent = Arrays.equals(original, decrypted);  // CHECK EQUIVALENCE
				
				if (!equivalent) {
					discrepencies++;
				}
			}
			
			// CHECK FOR ZERO DISCREPENCIES
			if (discrepencies == 0) {
				pass = true;
			}
			
//			// VISUALIZE DATA OUTPUT
//			for (byte[] b : decryptedPackets) {
//				String txt = new String(b);
//				System.out.println(txt);
//			}
		}
		
		// DISPLAY RESULT
		if (pass) {
			System.out.println("UNIT TEST [PASS / OK ] - [" + Test3_Streaming_Advanced_PacketLoss_Recovery.class.getCanonicalName() + "] -: ENCRYPTION / DECRYPTION SUCCESS");
		}
		else {
			System.err.println("UNIT TEST [FAIL] - [" + Test3_Streaming_Advanced_PacketLoss_Recovery.class.getCanonicalName() + "] -: ENCRYPTION / DECRYPTION FAILED");
		}
	}
	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** >> HELPER FUNCTIONS BELOW THIS LINE : 																																														  **/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	
	/**
	 * ATOMIC SUBROUTINE - INSPECT RESULTS OF ENCRYPTION - PRINT TO CONSOLE
	 * @param List<byte[]> src
	 * @param ArrayDeque<PacketVO> encryptedPackets
	 * @param Stopwatch sw
	 * @return void
	 * @throws Exception
	*/
	private static void inspectEncryptionResults(List<byte[]> src, ArrayDeque<PacketVO> encryptedPackets, Stopwatch sw) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			long totalOriginalSize = 0L;
			for (byte[] b : src) {
				totalOriginalSize += b.length;
			}
			
			long totalEncryptedSize = 0L;
			for (PacketVO p : encryptedPackets) {
				totalEncryptedSize += p.getData().length;
			}
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("ENCRYPTION RESULT: " + (encryptedPackets != null ? (encryptedPackets.size() + " ENCRYPTED PACKETS") : "NO RESULTS"));
			System.out.println("KEY MATERIALS:     " + (_SYMMETRIC_KEY_EXCHANGE != null ? _SYMMETRIC_KEY_EXCHANGE : "NO RESULTS"));
			System.out.println("CIPHERTEXT-LEN: [" + totalEncryptedSize + "]");
			System.out.println("ORIGINAL-LEN: [" + totalOriginalSize + "]");
			System.out.println("ENCRYPTION TIME: [" + sw.compareTimeMilli(1, 2) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	
	/**
	 * ATOMIC SUBROUTINE - INSPECT RESULTS OF DECRYPTION - PRINT TO CONSOLE
	 * @param List<byte[]> src
	 * @param List<byte[]> decryptedPackets
	 * @param Stopwatch sw
	 * @return void
	 * @throws Exception
	*/
	private static void inspectDecryptionResults(List<byte[]> src, List<byte[]> decryptedPackets, Stopwatch sw) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			long totalOriginalSize = 0L;
			for (byte[] b : src) {
				totalOriginalSize += b.length;
			}
			
			long totalDecryptedSize = 0L;
			for (byte[] b : decryptedPackets) {
				totalDecryptedSize += b.length;
			}
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("DECRYPTION RESULT: " + (decryptedPackets != null ? (decryptedPackets.size() + " DECRYPTED PACKETS") : "NO RESULTS"));
			System.out.println("DECRYPTED-LEN: [" + totalDecryptedSize + "]");
			System.out.println("ORIGINAL-LEN: [" + totalOriginalSize + "]");
			System.out.println("DECRYPTION TIME: [" + sw.compareTimeMilli(3, 4) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	
	/**
	 * ATOMIC SUBROUTINE - SIMULATE TEMP SENSOR DATA
	 * @return List<byte[]>
	*/
	private static List<byte[]> generateMockingDataSRC() {
		
		int sample_size = 100;
		
		// CREATE SAMPLE IIOT DATA SOURCE
		Random r = new Random();
		int max = 91;
		int min = 64;
		List<byte[]> src = new ArrayList<byte[]>();
		for (int i = 0; i < sample_size; i++) {
			String data = new String("TEMP [78] DEGREES");
			int adjust = r.nextInt(max - min) + min;
			data = data.replace("78", "" + adjust);
			src.add(data.getBytes());
		}
		
		return src;
	}
	
	/**
	 * ATOMIC SUBROUTINE - SIMULATE PACKET DROP - BASED ON THE SEQUENCE NUMBER OF A PACKET
	 * @param packetVO
	 * @return boolean
	*/
	private static boolean simulateDropPacket(PacketVO packetVO) {
		
		boolean drop = false;
		
		// SIMULATE PACKET-LOSS TYPE OF FAILURE SCENARIO
		if ((_DROP_PACKETS != null) && (_DROP_PACKETS.length > 0)) {
			
			for (int dropID : _DROP_PACKETS) {
				if (packetVO.getSequenceNumber() == dropID) {
					System.err.println("DROP PACKET NUMBER: " + packetVO.getSequenceNumber());
					drop = true;
					break;
				}
			}
		}
		
		return drop;
	}
	
	/**
	 * ATOMIC SUBROUTINE - CHECK IF A NUMBER FALLS WITHIN THE "DROP-PACKET" TEST SERIES
	 * @param number
	 * @return static
	*/
	private static boolean isDroppedPacketSimulation(int number) {
		
		boolean rval = false;
		
		for (int i : _DROP_PACKETS) {
			if (i == number) {
				rval = true;
				break;
			}
		}
		
		return rval;
	}
	
	/**
	 * INNER CLASS - DATA PACKET OBJECT (SIMULATION)
	*/
	private static class PacketVO implements Serializable {
		
		private static final long serialVersionUID = 3736078661217260849L;
		
		private long sequenceNumber;
		private byte[] data;
		
		/**
		 * @return the sequenceNumber
		 */
		public long getSequenceNumber() {
			return sequenceNumber;
		}
		/**
		 * @param sequenceNumber the sequenceNumber to set
		 */
		public void setSequenceNumber(long sequenceNumber) {
			this.sequenceNumber = sequenceNumber;
		}
		/**
		 * @return the data
		 */
		public byte[] getData() {
			return data;
		}
		/**
		 * @param data the data to set
		 */
		public void setData(byte[] data) {
			this.data = data;
		}
	}
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	 * @return void
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			Test3_Streaming_Advanced_PacketLoss_Recovery.test(); 	 // EXECUTE UNIT TEST
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
