package tests.part4;
/*********************************************************************************************
//* Filename:		Test3_Keytool_Create_HyperKey.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;

import com.clear.cryptosystem.CLEAR;
import com.clear.cryptosystem.CLEARKeyTool;
import com.clear.cryptosystem.CLEARLicenseInfo;
import com.clear.cryptosystem.CLEARMode.SingleJobDecryptionMode;
import com.clear.cryptosystem.CLEARMode.SingleJobEncryptionUserSuppliedKeyMode;
import com.clear.cryptosystem.CLEARResult;
import com.clear.cryptosystem.CLEARString;

import tools.EvaluateResult;
import tools.InspectResult;
import tools.LoadMFA;
import tools.Stopwatch;



public class Test3_Keytool_Create_HyperKey {

	// TEST DATA - HARD CODED
	private static final String _CLEARTEXT_TEST_DATA 			= "The quick brown fox jumps over the lazy dog!!  Some UTF-16 based 中国人 characters";
	
	private static final boolean _DISPLAY_RESULTS_TO_CONSOLE 	= true;  // SET FLAG TO TRUE TO INSPECT RESULTS / OUTPUT TO CONSOLE
	
	/**
	 * UNIT TEST BLOCK
	 * @param byte[] mfaMaterial
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean test(byte[] mfaMaterial) throws Exception {
		
		boolean pass = false;
		
		System.out.println("RUN UNIT TEST: [" + Test3_Keytool_Create_HyperKey.class.getCanonicalName() + "]");
		
		int strength = CLEAR.STRENGTH_512_BIT;
//		int strength = CLEAR.STRENGTH_1024_BIT;
//		int strength = CLEAR.STRENGTH_2048_BIT;
//		int strength = CLEAR.STRENGTH_5120_BIT;
//		int strength = CLEAR.STRENGTH_10240_BIT;
		
		
		// ENCRYPT SOME DATA WITH THE KEY THAT WAS CREATED
		// STOPWATCH - COMPARE TIMES FOR ENCRYPTION / DECRYPTION - DIFFERENT STRENGTH LEVELS
		Stopwatch sw = new Stopwatch();
		
		
		// MAKE A STANDARD-KEY
		CLEARKeyTool keyTool = CLEAR.clearKeyTool();
		String usk = keyTool.genKey(strength);
		
		
		// TRY ENCRYPTING SOMETHING WITH A HYPER KEY
		CLEARString cs = CLEAR.clearString();		// FACTORY PATTERN - GET A REFERENCE TO CLEAR API (STRING ENCRYPTION / DECRYPTION INTERFACE)
		
		
		// ENCRYPT A STRING - USER-SUPPLIED KEY
		sw.setTimeMark(1);
		CLEARResult cr = cs.encryptUSK(_CLEARTEXT_TEST_DATA, usk, SingleJobEncryptionUserSuppliedKeyMode.SINGLE_JOB_USER_SUPPLIED_KEY);
		sw.setTimeMark(2);
		
		// INSEPCT OUTPUT OF ENCRYPTION
		InspectResult.inspectEncryptionResultsString(cr, sw, _CLEARTEXT_TEST_DATA, Test3_Keytool_Create_HyperKey.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		// CONVERT STANDARD KEY TO A HYPER-KEY - MFA MATERIAL IS "OWNER MFA"
		String hyperKey = keyTool.createHyperKey(usk, mfaMaterial);
		
		
		
		// DECRYPT A STRING - USER-SUPPLIED HYPER-KEY
		sw.setTimeMark(3);
		cr = cs.decryptWithMFA(cr.getCipherTextString(), hyperKey, mfaMaterial, SingleJobDecryptionMode.SINGLE_JOB_KEY);
		sw.setTimeMark(4);
		
		
		
		// INSPECT OUTPUT OF DECRYPTION
		InspectResult.inspectDecryptionResultsString(cr, sw, _CLEARTEXT_TEST_DATA, Test3_Keytool_Create_HyperKey.class.getCanonicalName());   // DIAGNOSTIC TESTING ONLY - INCLUDES PERFORMANCE METRIC
		
		
		// EVALUATE DECRYPTION RESULT
		pass = EvaluateResult.evaluateStringDecryptionResult(cr, _CLEARTEXT_TEST_DATA, Test3_Keytool_Create_HyperKey.class.getCanonicalName());
		
		return pass;
	}
	
	/**
	 * SELF-EXECUTABLE - UNIT TEST 
	 * @param args
	*/
	public static void main(String[] args) {
		
		try {
			
			CLEARLicenseInfo cli = CLEAR.clearLicense();
			System.out.println(cli.getLicenseDetails());
			
			EvaluateResult.set_DISPLAY_TEST_RESULTS(_DISPLAY_RESULTS_TO_CONSOLE);
			InspectResult.set_DISPLAY_RESULTS_TO_CONSOLE(_DISPLAY_RESULTS_TO_CONSOLE);
			
			if ((args != null) && (args.length == 1) && (args[0] != null)) {
				
				File mfa = new File(args[0]);
				
				if ((mfa != null) && ((mfa.exists()))) {
					
					// MAKE A BACKUP COPY OF THE INPUT FILE SO THAT IT CAN BE USED DURING TESTING
					byte[] mfaMaterial = LoadMFA.load(mfa.getAbsolutePath());
					
					test(mfaMaterial); 	 // EXECUTE UNIT TEST
				}
				else {
					System.out.println("Could not locate MFA material:  " + args[0]);
				}
			}
			else {
				System.out.println("This unit test requires program input parameters");
			}
		}
		catch(Exception ex) {
			ex.printStackTrace();
		}
	}
}
