package tools;
/*********************************************************************************************
//* Filename:		EvaluateResult.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;
import java.util.Arrays;
import java.util.List;

import com.clear.cryptosystem.CLEARResult;

public abstract class EvaluateResult {

	private static boolean _DISPLAY_TEST_RESULTS = true;

	/**
	 * EXAMINE ENCRYPTION RESULTS FOR EXPECTED OUTPUT - DISPLAYS PASS OR FAIL
	 * @param CLEARResult cr
	 * @param String original
	 * @param String testClass
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean evaluateStringDecryptionResult(CLEARResult cr, String original, String testClass) throws Exception {
		return evaluateStringDecryptionResult(cr, original, testClass, false);
	}
	
	/**
	 * EXAMINE ENCRYPTION RESULTS FOR EXPECTED OUTPUT - DISPLAYS PASS OR FAIL
	 * @param CLEARResult cr
	 * @param String original
	 * @param String testClass
	 * @param boolean inverted - check for the negative test-case
	 * @return boolean
	 * @throws Exception
	 * @overload
	*/
	public static boolean evaluateStringDecryptionResult(CLEARResult cr, String original, String testClass, boolean inverted) throws Exception {
		
		boolean pass = false;
		
		// IS THE DECRYPTED RESULT IDENTITAL THE INPUT
		
		if ((cr != null) && (cr.getClearTextString() != null) && (cr.getClearTextString().equals(original))) {
			pass = true;
		}
		
		if (inverted) {
			pass = !pass;
		}
		
		// DISPLAY RESULT
		if (_DISPLAY_TEST_RESULTS) {
			if (pass) {
				System.out.println("UNIT TEST [PASS / OK ] - [" + testClass + "] -: ENCRYPTION / DECRYPTION SUCCESS");
			}
			else {
				System.err.println("UNIT TEST [FAIL] - [" + testClass + "] -: ENCRYPTION / DECRYPTION FAILED");
			}
		}
		
		return pass;
	}
	
	/**
	 * EXAMINE ENCRYPTION RESULTS FOR EXPECTED OUTPUT - DISPLAYS PASS OR FAIL
	 * @param CLEARResult cr
	 * @param String originalFileAbsolutePath
	 * @param String testClass
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean evaluateFileDecryptionResult(CLEARResult cr, String originalFileAbsolutePath, String testClass) throws Exception {
		
		boolean pass = false;
		
		// IS THE DECRYPTED RESULT IDENTITAL THE INPUT
		if ((cr != null) && (cr.getClearTextFile() != null)) {
			
			String cleartextFileResult = cr.getClearTextFile(); // EXAMINE RESULTS
			
			File originalFile = new File(originalFileAbsolutePath);
			File decryptedResult = new File(cleartextFileResult);
			
			// MAKE SURE BOTH ORIGINAL AND DECRYPTION RESULT FILES EXIST
			if (originalFile.exists() && decryptedResult.exists()) {
				
				// MAKE SURE FILE LENGTHS MATCH EXACTLY
				if (originalFile.length() == decryptedResult.length()) {
					
					String sha512_input = SHA512HashUtil.getSHA512(originalFile);
					String sha512_output = SHA512HashUtil.getSHA512(decryptedResult);
					
					if (sha512_input.equals(sha512_output)) {
						pass = true;
					}
				}
			}
		}
		
		// DISPLAY RESULT
		if (_DISPLAY_TEST_RESULTS) {
			if (pass) {
				System.out.println("UNIT TEST [PASS / OK ] - [" + testClass + "] -: ENCRYPTION / DECRYPTION SUCCESS");
			}
			else {
				System.err.println("UNIT TEST [FAIL] - [" + testClass + "] -: ENCRYPTION / DECRYPTION FAILED");
			}
		}
		
		return pass;
	}
	
	
	/**
	 * EXAMINE ENCRYPTION RESULTS FOR EXPECTED OUTPUT - DISPLAYS PASS OR FAIL
	 * @param List<byte[]> src
	 * @param List<byte[]> decryptedPackets
	 * @return boolean
	 * @throws Exception
	*/
	public static boolean evaluateUnitTestResultStreaming(List<byte[]> src, List<byte[]> decryptedPackets, String testClass) throws Exception {
		
		boolean pass = false;
		
		if ((src != null) && (decryptedPackets != null)) {
			
			
			if (src.size() == decryptedPackets.size()) {
				
				int discrepencies = 0;
				
				for (int i = 0; i < src.size(); i++) {
					
					byte[] original  = src.get(i);
					byte[] decrypted = decryptedPackets.get(i);
					
					boolean equivalent = Arrays.equals(original, decrypted);  // CHECK EQUIVALENCE
					
					if (!equivalent) {
						discrepencies++;
					}
				}
				
				// CHECK FOR ZERO DISCREPENCIES
				if (discrepencies == 0) {
					pass = true;
				}
			}
		}
		
		// DISPLAY RESULT
		if (_DISPLAY_TEST_RESULTS) {
			if (pass) {
				System.out.println("UNIT TEST [PASS / OK ] - [" + testClass + "] -: ENCRYPTION / DECRYPTION SUCCESS");
			}
			else {
				System.err.println("UNIT TEST [FAIL] - [" + testClass + "] -: ENCRYPTION / DECRYPTION FAILED");
			}
		}
		
		return pass;
	}
	

	/**
	 * @param _DISPLAY_TEST_RESULTS the _DISPLAY_TEST_RESULTS to set
	 */
	public static void set_DISPLAY_TEST_RESULTS(boolean _DISPLAY_TEST_RESULTS) {
		EvaluateResult._DISPLAY_TEST_RESULTS = _DISPLAY_TEST_RESULTS;
	}
}
