package tools;
/*********************************************************************************************
//* Filename:		FileIO.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

public abstract class FileIO {
	
	public static final int BUFFER_4K 				= 4096; // USED FOR FILE COPY

	public static final String _ERR_MSG_10003 		= "File not found: ";
	public static final String _NEWLINE 			= "\n";
	
	
	/**
	 * Copy a file A to file B
	 * @param srFile
	 * @param dtFile
	 * @return void
	 * @throws Exception
	*/
	public static void copyfile(File file1, File file2) throws Exception {
		
		  if (!file1.exists()) {
			  throw new Exception("Cannot copy file - source file not found!");
		  }
		  else {
			  InputStream in = null;
			  OutputStream out = null;
			  try {
				  in = new FileInputStream(file1);
				  if (file2.exists()) {
					  file2.delete(); // overwrite existing target file if it exists!
				  }
				  else {
					  file2.createNewFile(); // attempt to create file at pathname
				  }
				  out = new FileOutputStream(file2); // overwrite existing target file if it exists!
				  byte[] buf = new byte[BUFFER_4K];
			      int len;
			      while ((len = in.read(buf)) > 0){
			        out.write(buf, 0, len);
			      }  
			  }
			  finally {
				  if (in != null) {
					  in.close();
				  }
				  if (out != null) {
					  out.flush();
				      out.close(); 
				  }
			  }
		  }
	}
	
	/**
	 * Get File Extension Parts as String[]
	 * @param String fileName
	 * @return String[]
	*/
    public static String[] getFileExtensionParts(String fileName) {
        File file = new File(fileName);
        String[] result = new String[2];
        
        // Get the absolute path of the file without the extension
        int extensionIndex = fileName.lastIndexOf('.');
        if (extensionIndex > 0) {
            result[0] = file.getAbsolutePath().substring(0, extensionIndex);
            result[1] = fileName.substring(extensionIndex);
        } else {
            result[0] = file.getAbsolutePath();
            result[1] = "";
        }
        
        return result;
    }
    
	/**
	 * Returns the contents of a File as a java.lang.String object (if possible)
	 * @param File file
	 * @return Object
	 * @throws Exception
	*/
	public static String getStringFromTextFile(File file) throws Exception {
	
		String returnString = null;
	
		if (!file.exists()) {
			throw new Exception(_ERR_MSG_10003 + file.getName());
		}
		
		InputStream is = null;
		BufferedReader buf = null;
		
		try {
			is = new FileInputStream(file); 
			buf = new BufferedReader(new InputStreamReader(is));
			String line = buf.readLine();
			StringBuilder sb = new StringBuilder();
			while(line != null) {
				sb.append(line).append(_NEWLINE);
				line = buf.readLine();
			}
			returnString = sb.toString();
		}
		catch (Exception e) {
			throw new Exception(e.getMessage(),e.fillInStackTrace());
		}
		finally {
			try {
				if (is != null) {
					is.close();
					is = null;
				}
				if (buf != null) {
					buf.close();
					buf = null;
				}
			}
			catch(Exception ex) { }
		}
		return returnString;
	}
	
	/**
	 * Write the content of a String to a text file
	 * @param file
	 * @param content
	 * @throws Exception
	*/
	public static void writeStringToFileWithOverwrite(File file, String content) throws Exception {
		if (file.exists()) {
			file.delete();
		}
		file.createNewFile();
		Path targetPath = Paths.get(file.getAbsolutePath());
	    byte[] bytes = content.getBytes(StandardCharsets.UTF_8);
	    
	    Files.write(targetPath, bytes, StandardOpenOption.CREATE);
	}
}
