package tools;
/*********************************************************************************************
//* Filename:		InspectResult.java
//* ******************************************************************************************
//* 
//* 				SOFTWARE LICENSE AGREEMENT:
//* 				--------------------------------------------------------------------------
//* 				Licensed under the Apache License, Version 2.0 (the "License");
//* 				you may not use this file except in compliance with the License.
//* 				You may obtain a copy of the License at
//* 
//*    					https://www.apache.org/licenses/LICENSE-2.0
//* 
//* 				Unless required by applicable law or agreed to in writing, software
//* 				distributed under the License is distributed on an "AS IS" BASIS,
//* 				WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//* 				See the License for the specific language governing permissions and
//* 				limitations under the License.
//* 
//* ******************************************************************************************
//* 				
//* 				NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY - US DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				CRYPTOGRAPHIC MODULE VALIDATION PROGRAM (CVMP)
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is an *ACTIVE* FIPS-140-2 Validated Module.
//* 				--------------------------------------------------------------------------
//* 				CLEAR Cryptosystem is a standards-based cryptographic engine that enables 
//* 				the protection of data requiring absolute compliance with IS federal standards. 
//* 				
//* 				The module delivers core cryptographic functions for boosting the strength and 
//* 				speed of cryptographic protection while guaranteeing data integrity, data at 
//* 				rest encryption, and streaming data transmissions. 
//* 				
//* 				CLEAR Cryptosystem provides flexible modes of operation that include multi-factor
//* 				authentication (MFA) and embedded access control lists (ACL) for controlled 
//* 				access to data.
//* 				
//* 				https://csrc.nist.gov/projects/cryptographic-module-validation-program/certificate/4482
//* 				
//* ******************************************************************************************
//* 
//* 				
//* 				COMMODITY CLASSIFICATION : UNITED STATES DEPARTMENT OF COMMERCE
//* 				--------------------------------------------------------------------------
//* 				THIS ENCRYPTION ITEM PROVIDING AN OPEN CRYPTOGRAPHIC INTERFACE IS AUTHORIZED
//* 				FOR LICENSE EXCEPTION ENC UNDER SECTIONS 740.17 (A) AND (B)(2) OF THE EXPORT
//* 				ADMINISTRATION REGULATIONS (EAR). 
//* 				
//* 				UNITED STATES DEPARTMENT OF COMMERCE
//* 				BUREAU OF INDUSTRY AND SECURITY 
//* 				WASHINGTON, D.C. 20230
//* 				
//* 				BIS/EA/STC/IT
//* 
/********************************************************************************************/

import java.io.File;
import java.util.ArrayDeque;
import java.util.List;

import com.clear.cryptosystem.CLEARResult;

public abstract class InspectResult {

	private static boolean _DISPLAY_RESULTS_TO_CONSOLE = true;
	
	/**
	 * INSPECT RESULTS OF ENCRYPTION - PRINT TO CONSOLE
	 * @param CLEARResult cr
	 * @param Stopwatch sw
	 * @param String original
	 * @param String testClass
	 * @return void
	 * @throws Exception
	*/
	public static void inspectEncryptionResultsString(CLEARResult cr, Stopwatch sw, String original, String testClass) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			String ciphertextStringResult = cr.getCipherTextString(); // EXAMINE RESULTS
			String keyStringResult = cr.getKeyMaterial(); 
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("-- [ STRING ENCRYPTION ] RESULTS --: >>>> " + testClass);
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("ENCRYPTION RESULT: " + (ciphertextStringResult != null ? ciphertextStringResult : "NO RESULTS"));
			System.out.println("KEY MATERIALS:     " + (keyStringResult != null ? keyStringResult : "NO RESULTS"));
			System.out.println("CIPHERTEXT-LEN: [" + (ciphertextStringResult != null ? ciphertextStringResult.length() : "0") + "]");
			System.out.println("ORIGINAL-LEN:   [" + original.length() + "]");
			System.out.println("ENCRYPTION TIME: [" + sw.compareTimeMilli(1, 2) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	
	/**
	 * INSPECT RESULTS OF ENCRYPTION - PRINT TO CONSOLE
	 * @param CLEARResult cr
	 * @param Stopwatch sw
	 * @param String originalFileAbsolutePath
	 * @param String testClass
	 * @return void
	 * @throws Exception
	*/
	public static void inspectEncryptionResultsFile(CLEARResult cr, Stopwatch sw, String originalFileAbsolutePath, String testClass) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			String ciphertextFileResult = cr.getCipherTextFile(); // EXAMINE RESULTS
			String keyFileResult = cr.getKeyFile(); 
			
			File originalFile = new File(originalFileAbsolutePath);
			File ciphertextFile = new File(ciphertextFileResult);
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("-- [ FILE ENCRYPTION ] RESULTS --: >>>> " + testClass);
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("ENCRYPTION RESULT: " + (ciphertextFileResult != null ? ciphertextFileResult : "NO RESULTS"));
			System.out.println("KEY MATERIALS:     " + (keyFileResult != null ? keyFileResult : "NO RESULTS"));
			System.out.println("CIPHERTEXT-LEN: [" + ((ciphertextFile != null && ciphertextFile.exists()) ? ciphertextFile.length() : "0") + "]");
			System.out.println("ORIGINAL-LEN: [" + ((originalFile != null && originalFile.exists()) ? originalFile.length() : "0") + "]");
			System.out.println("ENCRYPTION TIME: [" + sw.compareTimeMilli(1, 2) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	
	/**
	 * INSPECT RESULTS OF ENCRYPTION - PRINT TO CONSOLE
	 * @param List<byte[]> src
	 * @param ArrayDeque<byte[]> encryptedPackets
	 * @param Stopwatch sw
	 * @param String _SYMMETRIC_KEY_EXCHANGE
	 * @param String testClass
	 * @return void
	 * @throws Exception
	*/
	public static void inspectEncryptionResultsStreaming(List<byte[]> src, ArrayDeque<byte[]> encryptedPackets, Stopwatch sw, String _SYMMETRIC_KEY_EXCHANGE, String testClass) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			long totalOriginalSize = 0L;
			for (byte[] b : src) {
				totalOriginalSize += b.length;
			}
			
			long totalEncryptedSize = 0L;
			for (byte[] b : encryptedPackets) {
				totalEncryptedSize += b.length;
			}
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("-- [ STREAMING ENCRYPTION ] RESULTS --: >>>> " + testClass);
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("ENCRYPTION RESULT: " + (encryptedPackets != null ? (encryptedPackets.size() + " ENCRYPTED PACKETS") : "NO RESULTS"));
			System.out.println("KEY MATERIALS:     " + (_SYMMETRIC_KEY_EXCHANGE != null ? _SYMMETRIC_KEY_EXCHANGE : "NO RESULTS"));
			System.out.println("CIPHERTEXT-LEN: [" + totalEncryptedSize + "]");
			System.out.println("ORIGINAL-LEN: [" + totalOriginalSize + "]");
			System.out.println("ENCRYPTION TIME: [" + sw.compareTimeMilli(1, 2) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/** 																																										**/
	/** 														DECRYPTION INSPECTION BELOW THIS LINE																			**/
	/** 																																										**/
	/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
	/**
	 * INSPECT RESULTS OF DECRYPTION - PRINT TO CONSOLE
	 * @param CLEARResult cr
	 * @param Stopwatch sw
	 * @param String original
	 * @param String testClass
	 * @return void
	 * @throws Exception
	*/
	public static void inspectDecryptionResultsString(CLEARResult cr, Stopwatch sw, String original, String testClass) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			String cleartextStringResult = cr.getClearTextString(); // EXAMINE RESULTS
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("-- [ STRING DECRYPTION ] RESULTS --: >>>> " + testClass);
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("DECRYPTION RESULT: " + (cleartextStringResult != null ? cleartextStringResult : "NO RESULTS"));
			System.out.println("DECRYPTED-LEN: [" + (cleartextStringResult != null ? cleartextStringResult.length() : "0") + "]");
			System.out.println("ORIGINAL-LEN:  [" + original.length() + "]");
			System.out.println("DECRYPTION TIME: [" + sw.compareTimeMilli(3, 4) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	
	/**
	 * INSPECT RESULTS OF DECRYPTION - PRINT TO CONSOLE
	 * @param CLEARResult cr
	 * @param Stopwatch sw
	 * @param String original
	 * @param String testClass
	 * @return void
	 * @throws Exception
	*/
	public static void inspectDecryptionResultsFile(CLEARResult cr, Stopwatch sw, String originalFileAbsolutePath, String testClass) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			String cleartextFileResult = cr.getClearTextFile(); // EXAMINE RESULTS
			
			File originalFile = new File(originalFileAbsolutePath);
			File decryptedResult = new File(cleartextFileResult);
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("-- [ FILE DECRYPTION ] RESULTS --: >>>> " + testClass);
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("DECRYPTION RESULT: " + (cleartextFileResult != null ? cleartextFileResult : "NO RESULTS"));
			System.out.println("DECRYPTED-LEN: [" + ((decryptedResult != null && decryptedResult.exists()) ? decryptedResult.length() : "0") + "]");
			System.out.println("ORIGINAL-LEN: [" + ((originalFile != null && originalFile.exists()) ? originalFile.length() : "0") + "]");
			System.out.println("DECRYPTION TIME: [" + sw.compareTimeMilli(3, 4) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	
	/**
	 * INSPECT RESULTS OF DECRYPTION - PRINT TO CONSOLE
	 * @param List<byte[]> src
	 * @param List<byte[]> decryptedPackets
	 * @param Stopwatch sw
	 * @param String testClass
	 * @return void
	 * @throws Exception
	*/
	public static void inspectDecryptionResultsStreaming(List<byte[]> src, List<byte[]> decryptedPackets, Stopwatch sw, String testClass) throws Exception {
		
		if (_DISPLAY_RESULTS_TO_CONSOLE) {
			
			long totalOriginalSize = 0L;
			for (byte[] b : src) {
				totalOriginalSize += b.length;
			}
			
			long totalDecryptedSize = 0L;
			for (byte[] b : decryptedPackets) {
				totalDecryptedSize += b.length;
			}
			
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("-- [ STREAMING DECRYPTION ] RESULTS --: >>>> " + testClass);
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("DECRYPTION RESULT: " + (decryptedPackets != null ? (decryptedPackets.size() + " DECRYPTED PACKETS") : "NO RESULTS"));
			System.out.println("DECRYPTED-LEN: [" + totalDecryptedSize + "]");
			System.out.println("ORIGINAL-LEN: [" + totalOriginalSize + "]");
			System.out.println("DECRYPTION TIME: [" + sw.compareTimeMilli(3, 4) + "] milliseconds");
			System.out.println("** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ");
			System.out.println("");
		}
	}
	

	/**
	 * @param _DISPLAY_RESULTS_TO_CONSOLE the _DISPLAY_RESULTS_TO_CONSOLE to set
	 */
	public static void set_DISPLAY_RESULTS_TO_CONSOLE(boolean _DISPLAY_RESULTS_TO_CONSOLE) {
		InspectResult._DISPLAY_RESULTS_TO_CONSOLE = _DISPLAY_RESULTS_TO_CONSOLE;
	}

	/**
	 * @return the _DISPLAY_RESULTS_TO_CONSOLE
	 */
	public static boolean is_DISPLAY_RESULTS_TO_CONSOLE() {
		return _DISPLAY_RESULTS_TO_CONSOLE;
	}
}
